<?php
if ( ! defined( 'WPINC' ) ) {
	die;
}

function vwi_settings_init() {
	// Register new settings for "vwi" page
	register_setting( 'vwi_vend_connect', 'vwi_vend_connect_subdomain' );
	register_setting( 'vwi_vend_connect', 'vwi_vend_connect_token' );
	register_setting( 'vwi_vend_connect', 'vwi_vend_connect_cashier' );
	register_setting( 'vwi_vend_import', 'vwi_vend_last_import_finish' );

	// register a new section in the "vwi" page
	add_settings_section(
		'vwi_settings_section_vend_connect',
		'VendHQ Connect Settings',
		'vwi_settings_section_vend_connect_cb',
		'vwi'
	);
	add_settings_section(
		'vwi_settings_section_vend_import',
		'VendHQ Import',
		'vwi_settings_section_vend_import_cb',
		'vwi-import'
	);

	// register a new field in the "vwi_settings_section_vend_connect" section, inside the "vwi" page
	add_settings_field(
		'vwi_field_subdomain', // as of WP 4.6 this value is used only internally
		// use $args' label_for to populate the id inside the callback
		'VendHQ Subdomain',
		'vwi_field_subdomain_cb',
		'vwi',
		'vwi_settings_section_vend_connect',
		[
			'label_for' => 'vwi_vend_connect_subdomain',
			'class'     => 'vwi_row',
		]
	);
	add_settings_field(
		'vwi_field_token', // as of WP 4.6 this value is used only internally
		// use $args' label_for to populate the id inside the callback
		'VendHQ Personal Token',
		'vwi_field_token_cb',
		'vwi',
		'vwi_settings_section_vend_connect',
		[
			'label_for' => 'vwi_vend_connect_token',
			'class'     => 'vwi_row',
		]
	);
	add_settings_field(
		'vwi_field_user', // as of WP 4.6 this value is used only internally
		// use $args' label_for to populate the id inside the callback
		'VendHQ Cashier User ID',
		'vwi_field_cashier_cb',
		'vwi',
		'vwi_settings_section_vend_connect',
		[
			'label_for' => 'vwi_vend_connect_cashier',
			'class'     => 'vwi_row',
		]
	);

	add_settings_field(
		'vwi_field_last_import_finish', // as of WP 4.6 this value is used only internally
		// use $args' label_for to populate the id inside the callback
		'VendHQ Last Import Completed',
		'vwi_field_last_import_finish_cb',
		'vwi-import',
		'vwi_settings_section_vend_import'
	);
}

/**
 * Custom option and settings:
 * callback functions
 */

// developers section cb
// section callbacks can accept an $args parameter, which is an array.
// $args have the following keys defined: title, id, callback.
// the values are defined at the add_settings_section() function.
function vwi_settings_section_vend_connect_cb( $args ) {
	?>
	<p id="<?php echo esc_attr( $args['id'] ); ?>">Settings for connecting to your VendHQ account</p>
	<p id="vwi_settings_section_vend_connect">
		<strong style="color:red">ATTENTION: </strong>
		If you already have the products created in WooCommerce then you will need a developer to attach them to the plugin for the plugin to be aware of them.
	</p>
	<p id="vwi_settings_section_vend_connect">
		<strong>QUESTIONS? </strong>
		Reach us
		<a href="https://sujith123.wufoo.com/forms/z1xchp690scddy2/" target="_blank">here</a>
	</p>
	<?php
}

function vwi_settings_section_vend_import_cb( $args ) {
	?>
	<p id="<?php echo esc_attr( $args['id'] ); ?>">Click the button to run import</p>
	<?php
}

// Fields cb
// field callbacks can accept an $args parameter, which is an array.
// $args is defined at the add_settings_field() function.
// WordPress has magic interaction with the following keys: label_for, class.
// the "label_for" key value is used for the "for" attribute of the <label>.
// the "class" key value is used for the "class" attribute of the <tr> containing the field.
// you can add custom key value pairs to be used inside your callbacks.
function vwi_field_subdomain_cb( $args ) {
	// get the value of the setting we've registered with register_setting()
	$option = get_option( 'vwi_vend_connect_subdomain' );
	?>
	<input type="text" name="<?php echo esc_attr( $args['label_for'] ); ?>" id="<?php echo esc_attr( $args['label_for'] ); ?>" value="<?php echo esc_attr( isset( $option ) ? $option : '' ); ?>"/>
	<p class="description">
		If your VendHQ address is <b>xyz</b>.vendhq.com, then <b>xyz</b> is your VendHQ subdomain
	</p>
	<?php
}

function vwi_field_token_cb( $args ) {
	$option = get_option( 'vwi_vend_connect_token' );
	?>
	<input type="text" name="<?php echo esc_attr( $args['label_for'] ); ?>" id="<?php echo esc_attr( $args['label_for'] ); ?>" value="<?php echo esc_attr( isset( $option ) ? $option : '' ); ?>"/>
	<p class="description">
		You can get Personal Token from VendHQ admin. Go to <b>Setup -> Personal Tokens -> Generate Personal Token</b>.
		After saving copy generated <b>Access token</b> to this field
	</p>
	<?php
}

function vwi_field_cashier_cb( $args ) {
	$option = get_option( 'vwi_vend_connect_cashier' );
	?>
	<input type="text" name="<?php echo esc_attr( $args['label_for'] ); ?>" id="<?php echo esc_attr( $args['label_for'] ); ?>" value="<?php echo esc_attr( isset( $option ) ? $option : '' ); ?>"/>
	<p class="description">
		This is the VendHQ User ID to which the sales from WooCommerce needs to be registered to. Go to <b>Setup ->
			Users -> Click on the user you need to select from the list of users</b>. Now the URL will contain the User
		ID.
	</p>
	<p class="description">
		If the URL is https://subdomain.vendhq.com/users/view/<b>daf7f240-accj-11gt-eddc-1bdklskmfo02</b>, then User ID
		is <b>daf7f240-accj-11gt-eddc-1bdklskmfo02</b>
	</p>
	<?php
}

function vwi_field_last_import_finish_cb() {
	$option = get_option( 'vwi_vend_last_import_finish' );
	?>
	<p class="description">
		<?php
		if ( $option ) {
			echo esc_html( human_time_diff( $option, current_time( 'timestamp' ) ) . ' ago' );
		} else {
			echo 'Never';
		}
		?>
	</p>
	<?php
}

function vwi_options_page() {
	add_menu_page(
		'VendHQ Integration',
		'Vend Integration',
		'manage_options',
		'vwi',
		'vwi_options_page_html'
	);
}

function vwi_options_page_html() {
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}

	// add error/update messages
	// check if the user have submitted the settings
	// WordPress will add the "settings-updated" $_GET parameter to the url
	$last_import = get_option( 'vwi_vend_last_import_finish' );
	$diff        = (int) abs( current_time( 'timestamp' ) - $last_import );
	if ( $diff < 5 ) {
		// add settings saved message with the class of "updated"
		add_settings_error( 'vwi_messages', 'vwi_message', 'Import Completed Successfully', 'updated' );
	} elseif ( isset( $_GET['settings-updated'] ) ) {
		add_settings_error( 'vwi_messages', 'vwi_message', 'Settings Saved', 'updated' );
	}

	// show error/update messages
	settings_errors( 'vwi_messages' );
	?>
	<div class="wrap">
		<h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
		<form action="options.php" method="post">
			<?php
			// output security fields for the registered setting "vwi"
			settings_fields( 'vwi_vend_connect' );
			// output setting sections and their fields
			// (sections are registered for "vwi", each field is registered to a specific section)
			do_settings_sections( 'vwi' );
			// output save settings button
			submit_button( 'Save Settings' );
			?>
		</form>
		<?php
		if ( get_option( 'vwi_vend_connect_subdomain' ) && get_option( 'vwi_vend_connect_token' ) ) {
			?>
			<form method="post" action="options.php">
				<?php
				// output security fields for the registered setting "vwi"
				settings_fields( 'vwi_vend_import' );
				// output setting sections and their fields
				// (sections are registered for "vwi", each field is registered to a specific section)
				do_settings_sections( 'vwi-import' );
				// output save settings button
				$other_attributes = array( 'onclick' => 'alert("This free version of the plugin is capable of importing only around 100 products in a single run. If you try to import more products it will fail during its execution but you can see the initial products imported in Products page. You may import manually repeatedly to get all the remaining products. The Premium version of the plugin do not have this limitation.")' );
				submit_button( 'Import products to WooCommerce', 'button-primary button-large', 'import', true, $other_attributes );
				?>
			</form>
			<?php
		}
		?>
	</div>
	<?php
}


