<?php
if ( ! defined( 'WPINC' ) ) {
	die;
}

function vwi_send_customer_and_sale( $order_id ) {
	$subdomain      = get_option( 'vwi_vend_connect_subdomain' );
	$api_key        = get_option( 'vwi_vend_connect_token' );
	$cashier        = get_option( 'vwi_vend_connect_cashier' );
	$create_contact = true;

	$order             = new WC_Order( $order_id );
	$email             = $order->get_billing_email();
	$first_name        = $order->get_billing_first_name();
	$last_name         = $order->get_billing_last_name();
	$phone             = $order->get_billing_phone();
	$company           = $order->get_billing_company();
	$billing_address   = $order->get_billing_address_1();
	$billing_address2  = $order->get_billing_address_2();
	$billing_city      = $order->get_billing_city();
	$billing_state     = $order->get_billing_state();
	$billing_postcode  = $order->get_billing_postcode();
	$billing_country   = $order->get_billing_country();
	$shipping_address  = $order->get_shipping_address_1();
	$shipping_address2 = $order->get_shipping_address_2();
	$shipping_city     = $order->get_shipping_city();
	$shipping_state    = $order->get_shipping_state();
	$shipping_postcode = $order->get_shipping_postcode();
	$shipping_country  = $order->get_shipping_country();

	$lower_fname      = strtolower( $first_name );
	$lower_lname      = strtolower( $last_name );
	$vend_customer_id = '';

	try {

		$args          = array(
			'headers' => array(
				'accept'        => 'application/json',
				'authorization' => "Bearer $api_key",
				'cache-control' => 'no-cache',
				'content-type'  => 'application/json',
			),
		);
		$search_result = vwi_api_request( 'https://' . $subdomain . ".vendhq.com/api/2.0/search?type=customers&first_name=$lower_fname&last_name=$lower_lname", 'GET', $api_key, $args );
		$search_result = ( json_decode( $search_result, true ) );
		$search_result = $search_result['data'];

		foreach ( $search_result as $result ) {
			if ( $result['email'] == $email ) {
				$create_contact   = false;
				$vend_customer_id = $result['id'];
				break;
			}
		}

		// create contact
		if ( $create_contact ) {
			$create_body     = array(
				'first_name'          => $first_name,
				'last_name'           => $last_name,
				'email'               => $email,
				'company_name'        => $company,
				'phone'               => $phone,
				'physical_address_1'  => $billing_address,
				'physical_address_2'  => $billing_address2,
				'physical_city'       => $billing_city,
				'physical_postcode'   => $billing_postcode,
				'physical_state'      => $billing_state,
				'physical_country_id' => $billing_country,
				'postal_address_1'    => $shipping_address,
				'postal_address_2'    => $shipping_address2,
				'postal_city'         => $shipping_city,
				'postal_postcode'     => $shipping_postcode,
				'postal_state'        => $shipping_state,
				'postal_country_id'   => $shipping_country,
			);
			$args            = array(
				'headers' => array(
					'accept'        => 'application/json',
					'authorization' => "Bearer $api_key",
					'cache-control' => 'no-cache',
					'content-type'  => 'application/json',
				),
				'body'    => json_encode( $create_body ),
			);
			$create_response = vwi_api_request( 'https://' . $subdomain . '.vendhq.com/api/2.0/customers', 'POST', $api_key, $args );

			$customers_response = ( json_decode( $create_response, true ) );
			$vend_customer_id   = $customers_response['data']['id'];
		}

		$items                  = $order->get_items();
		$register_sale_products = array();
		foreach ( $items as $item ) {
			$product_id                     = $item['product_id'];
			$product_qty                    = $item['qty'];
			$total_with_or_without_discount = $item['total'];
			$item_unit_price                = $total_with_or_without_discount / $product_qty;
			$vend_product_id_in_woo         = get_post_meta( $product_id, '_woo_vend_product_id', true );

			if ( ! $vend_product_id_in_woo ) {
				continue;
			}

			$register_sale_products[] = array(
				'product_id' => $vend_product_id_in_woo,
				'price'      => $item_unit_price,
				'quantity'   => $product_qty,
			);
		}

		if( empty( $register_sale_products ) ) {
			return;
		}

		// create sale
		$create_body = array(
			'user_id'                => $cashier,
			'status'                 => "ONACCOUNT_CLOSED",
			'customer_id'            => $vend_customer_id,
			'source_id'              => "woocommerce",
			'register_sale_products' => $register_sale_products,
		);
		$args        = array(
			'headers' => array(
				'accept'        => 'application/json',
				'authorization' => "Bearer $api_key",
				'cache-control' => 'no-cache',
				'content-type'  => 'application/json',
			),
			'body'    => json_encode( $create_body ),
		);
		vwi_api_request( "https://" . $subdomain . ".vendhq.com/api/register_sales", 'POST', $api_key, $args );
	} catch ( Exception $ex ) {
		echo $ex->getMessage();
		die();
	}
}