<?php

function vwi_run_import_from_vend() {

	$subdomain         = get_option( 'vwi_vend_connect_subdomain' );
	$api_key           = get_option( 'vwi_vend_connect_token' );
	$products_per_page = 1000;
	$after             = '';
	global $wpdb;

	$fetch_again = true;

	while ( $fetch_again ) {

		$args     = array(
			'headers' => array(
				'accept'        => 'application/json',
				'authorization' => "Bearer $api_key",
				'cache-control' => 'no-cache',
				'content-type'  => 'application/json',
			),
		);
		$response = vwi_api_request( "https://{$subdomain}.vendhq.com/api/2.0/products?page_size={$products_per_page}&{$after}", 'GET', $api_key, $args );

		$products_response = ( json_decode( $response, true ) );

		$after = 'after=' . $products_response['version']['max'];

		if ( count( $products_response['data'] ) < $products_per_page ) {
			$fetch_again = false;
		}

		foreach ( $products_response['data'] as $individual_product ) {
			if ( 'SYSTEM' == $individual_product['source'] ) {
				continue;
			}

			$version                = $individual_product['version'];
			$existing_wc_product_id = wc_get_product_id_by_sku( $individual_product['sku'] );
			$update_product         = false;
			$current_vend_version   = intval( get_post_meta( $existing_wc_product_id, '_current_vend_version', true ) );
			if ( $existing_wc_product_id && $current_vend_version != $version ) {
				$update_product = true;
			} elseif ( $existing_wc_product_id || ! $individual_product['is_active'] ) {
				// No update needed OR no need to process.
				continue;
			}

			$id                = trim( $individual_product['id'] );
			$sku               = $individual_product['sku'];
			$title             = $individual_product['name'];
			$title             = str_replace( '"', '\"', $title );
			$price             = $individual_product['price_including_tax'];
			$image_url         = $individual_product['image_url'];
			$images            = $individual_product['images'];
			$has_variant       = $individual_product['has_variants'];
			$description       = $individual_product['description'];
			$description       = str_replace( '"', '\"', $description );
			$has_inventory     = $individual_product['has_inventory'];
			$variant_parent_id = $individual_product['variant_parent_id'];
			$tags_vend         = $individual_product['categories'];
			$category_vend     = $individual_product['type'];

			$total_inventory           = 0;
			$gallery                   = array();
			$current_images_in_vend    = array();
			$already_imported_images   = array();
			$currently_imported_images = array();
			$imported_images           = array();

			// using the product id to find total inventory count and other details not in the products API
			$args     = array(
				'headers' => array(
					'accept'        => 'application/json',
					'authorization' => "Bearer $api_key",
					'cache-control' => 'no-cache',
					'content-type'  => 'application/json',
				),
			);
			$response = vwi_api_request( "https://{$subdomain}.vendhq.com/api/1.0/product/{$id}", 'GET', $api_key, $args );

			$product_response = ( json_decode( $response, true ) );

			if ( isset( $product_response['inventory'] ) && is_array( $product_response['inventory'] ) ) {
				foreach ( $product_response['inventory'] as $inventory ) {
					$total_inventory += intval( $inventory['count'] );
				}
			}

			if ( ! $has_variant && ! $variant_parent_id ) { // Simple products
				if ( $update_product ) {
					$product = new WC_Product_Simple( $existing_wc_product_id );
					if ( ! $individual_product['is_active'] ) {
						// Delete product if inactive.
						$product->delete( true );
						$wpdb->query( 'COMMIT' );
						continue;
					}
					$already_imported_images = get_post_meta( $existing_wc_product_id, '_imported_images_versions', true );
					$already_imported_images = json_decode( $already_imported_images, true );
					if( $already_imported_images == NULL ) {
						$already_imported_images = [];
					}
					$gallery                 = $product->get_gallery_image_ids();
				} else {
					$product = new WC_Product_Simple();
				}
				
				// Start MySQL individual product transaction
				$wpdb->query( 'START TRANSACTION' );

				// Manage terms.
				if ( $category_vend ) {
					$term = term_exists( $category_vend['name'], 'product_cat' );
					if ( empty( $term ) ) {
						$term = wp_insert_term( $category_vend['name'], 'product_cat' );
					}
					add_term_meta( $term['term_id'], 'vend_id', $category_vend['id'], true );
					$product->set_category_ids( [ $term['term_id'] ] );
				} else {
					$product->set_category_ids( array() );
				}

				$tag_ids = array();
				foreach ( $tags_vend as $tag_vend ) {
					$term = term_exists( $tag_vend['name'], 'product_tag' );
					if ( empty( $term ) ) {
						$term = wp_insert_term( $tag_vend['name'], 'product_tag' );
					}
					add_term_meta( $term['term_id'], 'vend_id', $tag_vend['id'], true );
					$tag_ids[] = $term['term_id'];
				}
				$product->set_tag_ids( $tag_ids );

				$product->set_name( $title );
				$product->set_sku( $sku );
				$product->set_description( $description );
				$product->set_regular_price( $price );

				if ( $has_inventory ) {
					$product->set_manage_stock( true );
					$product->set_stock_status( 'instock' );
					$product->set_stock_quantity( intval( $total_inventory ) );
				}
				$product->save(); // Saving here so that the images uploaded below will have an attached product in the Media Library. Else the images will be shown as unattached.

				foreach ( $images as $key => $image ) {
					if ( $update_product ) {
						$current_images_in_vend[] = $image['version'];
						if ( in_array( $image['version'], $already_imported_images ) ) {
							continue;
						}
					}

					$upload = wc_rest_upload_image_from_url( esc_url_raw( $image['url'] ) );
					if ( is_wp_error( $upload ) ) {
						throw new Exception( 'Vend main image upload error.' );
					}

					$attachment_id = wc_rest_set_uploaded_image_as_attachment( $upload, $product->get_id() );

					if ( ! wp_attachment_is_image( $attachment_id ) ) {
						throw new Exception( sprintf( '#%s is an invalid image ID.', $attachment_id ) );
					}

					if ( isset( $image_url ) && $image['sizes']['standard'] == $image_url ) {
						$product->set_image_id( $attachment_id );
						// Set the image name
						wp_update_post( array(
							'ID'         => $attachment_id,
							'post_title' => $title . " Main Image",
						) );
					} else {
						$gallery[] = $attachment_id;
						// Set the image name
						wp_update_post( array(
							'ID'         => $attachment_id,
							'post_title' => $title . " Gallery Image " . $key,
						) );
					}

					// Save for future reference.
					$currently_imported_images[ $attachment_id ] = $image['version'];
				}

				foreach ( $already_imported_images as $key => $value ) {
					if ( ! in_array( $value, $current_images_in_vend ) ) {
						if ( false === wp_delete_attachment( $key ) ) {
							throw new Exception( sprintf( 'Delete attachment with ID %s failed.', $key ) );
						}
						unset( $already_imported_images[ $key ] );
						$gallery = array_diff( $gallery, array( $key ) );
					}
				}

				$product->set_gallery_image_ids( $gallery );
				$product->save();

				update_post_meta( $product->get_id(), '_woo_vend_product_id', $id );
				update_post_meta( $product->get_id(), '_current_vend_version', $version );
				if ( ! empty( $already_imported_images ) ) {
					$imported_images = array_replace( $already_imported_images, $currently_imported_images );
				} else {
					$imported_images = $currently_imported_images;
				}
				$imported_images = json_encode( $imported_images );
				update_post_meta( $product->get_id(), '_imported_images_versions', $imported_images );
				$wpdb->query( 'COMMIT' );
			}

			echo esc_html( 'Import/Update Completed - ' . $individual_product['name'] . '<br>' );
		}
	}
	update_option( 'vwi_vend_last_import_finish', current_time( 'timestamp' ), false );
}

function vwi_import_from_vend() {
	global $wpdb;
	try {
		set_time_limit( 0 );
		vwi_run_import_from_vend();
	} catch ( Exception $ex ) {
		$wpdb->query( 'ROLLBACK' );
		echo $ex->getMessage();
		die();
	}

}
